/* fcolor.c -- read TGA file containing 24-bit integer values, and
 *             convert via HSL function to 24-bit red+green+blue values.
 *
 * Original Author: Tim Wegner Dec. ? 1995
 * Modifications:   John Beale Dec. 6 1995 <beale@jump.stanford.edu>
 */

#include <stdio.h>
#include <math.h>

#define D double
#define Byte unsigned char

void HSL_to_RGB(D h,D s,D l,D *r,D *g,D *b); /* useful real-val. transform */
void rgbmap(long maxiter, long iter, Byte *r, Byte *g, Byte *b);

double par[9] = {2, .2, 0.3,   /* hue range, offset, exponent */
                -.5, 1, 0.3,   /* sat "                       */
                 2,  0, 0.2};  /* luminosity "                */

void rgbmap(long maxiter, long iter, Byte *r, Byte *g, Byte *b)
{
 D hue,sat,lum;
 D red,grn,blu;
 
 /* printf("%ld\n",iter); */
 hue = pow(((D)(iter)/maxiter),par[2]) * par[0] + par[1];
 sat = pow(((D)(iter)/maxiter),par[5]) * par[3] + par[4];
 lum = pow(((D)(iter)/maxiter),par[8]) * par[6] + par[7];
 HSL_to_RGB(hue,sat,lum,&red,&grn,&blu);
 *r = (Byte)(blu * 255.99);
 *g = (Byte)(grn * 255.99);
 *b = (Byte)(red * 255.99);
}

void main(int argc, char **argv)
{
   int xdots, ydots, i, j, err, n;
   long iter, maxiter;
   Byte red, green, blue;
   FILE *fpin, *fpout;
   char fin[80] = "iterates.tga";
   char fout[80] = "new.tga";
   char buf1[12];
   char buf2[2];
   char *usage = "Usage: fcolor <fin> <fout> [h1,2,3 s1,2,3 l1,2,3]\n\
        where the nine optional numerical parameters control \n\
        hue1 -- hue scaling factor (1.0 == full spectrum, once)\n\
        hue2 -- hue starting offset (0 = red, .45 = green, .9 = blue\n\
        hue3 -- hue exponent: fiter=[0..1]; pow(fiter,hue3) * hue1 + hue2\n\
        sat1 -- saturation scaling factor (1.0 == fully saturated, 0 = grey\n\
        sat2 -- starting saturation value\n\
        sat3 -- saturation exponent\n\
        lum1 -- luminosity scaling factor (1.0 == brightest, 0.0 = black\n\
        lum2 -- starting luminosity value\n\
        lum3 -- luminosity exponent\n";

   if ((argc < 3) || (argc > 12) ) {
      fprintf(stderr,"%s",usage);
      exit(1);
   }
   strcpy(fin,argv[1]);
   strcpy(fout,argv[2]);
   for(n=3;n<argc;n++) {
     sscanf(argv[n],"%lf",&par[n-3]); 
/*     printf("%d:%f ",n,par[n-3]);
     fflush(stdout); */
   }
   /* printf("\n");       */

   if((fpin = fopen(fin,"rb"))==NULL)
   {
      fprintf(stderr,"Can't open %s\n",fin);
      exit(1);
   }
   if((fpout = fopen(fout,"wb"))==NULL)
   {
      fprintf(stderr,"Can't open %s\n",fout);
      exit(1);
   }

   fread (buf1,   12,1,fpin);    /* read 12 bytes */
   fread (&xdots,  2,1,fpin);    /* read xdots */
   fread (&ydots,  2,1,fpin);    /* read ydots */
   fread (buf2,    2,1,fpin);    /* read 2 bytes */
   fread (&maxiter,4,1,fpin);    /* read 4 bytes */
   buf1[0] = 0; /* we're not writing maxiter */
   fwrite (buf1,  12,1,fpout);   /* write 12 bytes */
   fwrite (&xdots, 2,1,fpout);   /* write xdots */
   fwrite (&ydots, 2,1,fpout);   /* write ydots */
   fwrite (buf2,   2,1,fpout);   /* write 2 bytes */

/*   maxiter = 65535; */
   printf("xdots %d ydots %d maxiter %ld\n",xdots,ydots,maxiter);
   printf("Hue %3.2f %3.2f %3.2f Sat %3.2f %3.2f %3.2f Lum %3.2f %3.2f %3.2f\n",
        par[0],par[1],par[2],par[3],par[4],par[5],par[6],par[7],par[8]);
   for(j=0;j<ydots;j++)
   {
      printf("row %2d\r",j);
      for(i=0;i<xdots;i++)
      {
         iter = 0;
         if((err=fread(&iter,3,1,fpin))==0)
            exit(1);
         /* map iterations to colors */
         rgbmap(maxiter,iter,&red,&green,&blue); 
         fwrite(&blue, 1,1,fpout);
         fwrite(&green,1,1,fpout);
         fwrite(&red,  1,1,fpout);
      }
   }
}



